==> hakk_builder-0.2.1.tar.gz: lib/src/basic/_mmu.dart <==
part of hakkit.hakk.builder.basic;

// memory manager for the compiler
class _MMU{
  static final List<_MemUnit> memoryPool = new List<_MemUnit>();
  
  static void addMemoryRange(String name, int startRange, int endRange){
    if (memoryPool.any((m) => m.blockName == name)){
      throw 'Cannot add memory to the MMU that is already defined with name "${name}"';
    }
    
    _log.fine('Adding $name to memory pool, starting at $startRange, ending $endRange');
    for(int i = startRange; i < endRange; i++){
      memoryPool.add(new _MemUnit(name, i));    
    }
  }
  
  // deallocates memory for the given context back to the memory pool.
  static void dalloc(String context){
    _log.fine('Attempting to dalloc() $context');
    memoryPool
      .where((m) => m.context == context)
      .forEach((m){
        _log.fine('   deallocating ${m.blockName}${m.index} ${m.context}');
        m.context = '';
        m.allocated = false;
      });
  }
  
  // attempts to allocate a memory unit from the pool.
  static _HakkObject malloc([String context = '']){
    final mu = memoryPool.firstWhere((m) => !m.allocated, orElse: () => null);
    if (mu == null){
      throw 'Compiler ran out of memory.  You will need to add another Block.system()'
      ' to your code in order to compile this hakk.';
    }
       
    mu.context = context;
    mu.allocated = true;
    
    _log.fine('malloc() ${mu.context} - ${mu.reference}');
    
    return mu.reference;
  }
  
  static void clear(){
    memoryPool.clear();
  }
}

==> hakk_builder-0.2.2.tar.gz: lib/src/basic/_mmu.dart <==
part of hakkit.hakk.builder.basic;

// memory manager for the compiler
class _MMU{
  static final List<_MemUnit> memoryPool = new List<_MemUnit>();
  static int gcRuns = 0;
  
  static const int GC_THRESHOLD = 90; //gc's when memory allocated over this %
  
  static int get percentAllocated => ((memoryPool.where((m) => m.allocated).length / memoryPool.length) * 100).round();
  
  static void addMemoryRange(String name, int startRange, int endRange){
    if (memoryPool.any((m) => m.blockName == name)){
      throw 'Cannot add memory to the MMU that is already defined with name "${name}"';
    }
    
    _log.fine('Adding $name to memory pool, starting at $startRange, ending $endRange');
    for(int i = startRange; i < endRange; i++){
      memoryPool.add(new _MemUnit(name, i));    
    }
  }
  
  // attempts to allocate a memory unit from the pool.
  static _HakkObject malloc([String context = '']){
    if (percentAllocated > GC_THRESHOLD){
      _gc();
    }
    
    final mu = memoryPool.firstWhere((m) => !m.allocated, orElse: () => null);
    if (mu == null){
      throw 'Compiler ran out of memory.  You will need to add another Block.system()'
      ' to your code in order to compile this hakk.';
    }
       
    mu.context = context;
    mu.allocated = true;
    
    _log.fine('malloc() ${mu.blockName}${mu.index} ${mu.context}');
    
    return mu.reference;
  }
  
  // deallocates memory for the given context back to the memory pool.
  static void dalloc(String context){
    _log.fine('Attempting to dalloc() MUs in context: $context');
        
    memoryPool
      .where((m) => m.context == context)
      .forEach((m){
        _log.fine('   marking ${m.blockName}${m.index} ${m.context} for GC');
        m.context = '';
        m.canGC = true;
      });
  }
   
  static void _gc(){   
    int acc = 0;
    memoryPool
      .where((m) => m.canGC)
      .forEach((m){
        m.allocated = false;
        m.canGC = false;
        ++acc;
      });
    
    ++gcRuns;
    _log.fine('GC released ${acc} MUs');
    
    if (acc == 0){
      _log.warning('GC failed to release any MUs back to the pool. ${percentAllocated}% allocated.');
    }
  }
  
  static void clear(){
    memoryPool.clear();
    gcRuns = 0;
  }
}
